<link rel="stylesheet" href="{{ STATIC_URL }}css/jquery.graphviz.css" />
<script type="text/javascript" src="{{ STATIC_URL }}js/jquery-1.7.2.min.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}js/jquery-ui-1.8.21.min.js"></script>
<script type="text/javascript" src="{{ STATIC_URL }}js/jquery.graphviz.js"></script>
